#!/bin/bash

#INSTRUCTIONS: in order for the program to work you need to run it in your terminal once you are inside the folder "Ophiodon_elongatus" downloaded from our web


#QUERY ORGANIZATION: We divide the original files (selenoprotDR.fa and cisteinaDR.fa) into different files containing one protein each
sed -r 's/^>[^(]+\(([^)]+)\).*/>\1/' originals/selenoprotDR.fa > originals/temp.fa; mv originals/temp.fa originals/selenoprotDR.fa
cat originals/selenoprotDR.fa | awk '{
  if (substr($0, 1, 1)==">") {filename=(substr($0,2,length($0)) ".fa")}
  print $0 > filename
}'

sed -r 's/^>[^(]+\(([^)]+)\).*/>\1/' originals/cisteinaDR.fa > originals/temp.fa; mv originals/temp.fa originals/cisteinaDR.fa
cat originals/cisteinaDR.fa | awk '{
  if (substr($0, 1, 1)==">") {filename=(substr($0,2,length($0)) ".fa")}
  print $0 > filename
}'


#FILE LOOP: for each file, we run this loop to automatically put the reference protein file inside a folder (named after it) and analyse the protein
for file in *.fa
do
	foldername="${file%.*}"
	mkdir ${foldername}
	mv ${file} ${foldername}

	# We remove the unwanted symbols and change the U present in the sequence for X
	sed -r 's/U/X/g; s/[#%@]//g' -i ${foldername}/${file}

  #TBLASTN: we align the query sequences of Danio rerio with the genome sequence of Ophiodon elongatus
  echo "${foldername}#################################################################################################################################################################################################"
  echo "TBLASTN======================================================================================================================================================================================================="
  tblastn -query ${foldername}/$file -db /mnt/NFS_UPF/soft/genomes/2021/Ophiodon_elongatus/genome.fa -outfmt 7 -out ${foldername}/${foldername}_tblastn.out
  cat ${foldername}/${foldername}_tblastn.out

  #SCAFFOLD LOOP: we select the scaffolds that meet the criteria and, for each of them, we reun the programs to predict the protein and align it with the query sequences of Danio rerio
  grep -v "#" ${foldername}/${foldername}_tblastn.out | awk '{if ($3>30 && $11<0.0001) {print $2}}' | sort | uniq | while read scaffold;
  do
    #We create a folder inside the "protein folder" named after the scaffold, where we will save all the generated data
    echo "$scaffold//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////"
    mkdir ${foldername}/${scaffold}

    #FASTAFETCH: We isolate the studied scaffold from the others
    fastafetch /mnt/NFS_UPF/soft/genomes/2021/Ophiodon_elongatus/genome.fa /mnt/NFS_UPF/soft/genomes/2021/Ophiodon_elongatus/genome.index ${scaffold} > ${foldername}/${scaffold}/${foldername}_${scaffold}_fastafetch.fa

    #We select the lowest start point of the alignment in the scaffold and, then, we substract 50000 to this number to get the starting point of the fastasubseq. In case of getting a nevative number, we start at position 0
    min1=$(grep "${scaffold}" ${foldername}/${foldername}_tblastn.out | cut -f 9 | sort -n | head -1)
    min2=$(grep "${scaffold}" ${foldername}/${foldername}_tblastn.out | cut -f 10 | sort -n | head -1)

    if [[ $min1 -gt $min2 ]]; then
      start=$min2
    elif [[ $min1 -lt $min2 ]]; then
      start=$min1
    fi

    if [[ $min1 -gt $min2 ]]; then
      strand="reverse"
    elif [[ $min1 -lt $min2 ]]; then
      strand="forward"
    fi

    if [[ ${start} -gt 50000 ]]; then
      start=$((${start}-50000))
    else
      start=0
    fi
    echo $start

    #We establish the length of the fastasubseq to 100000. In case that tha scaffold ends before this length, we establish the length so as the fastasubseq output finishes at the end point of the scaffold
    scaffold_length=$(grep ${scaffold} /mnt/NFS_UPF/soft/genomes/2021/Ophiodon_elongatus/genome.lengths | cut -f 1 -d " ")
    if [[ ${scaffold_length} -gt $((${start}+100000)) ]]; then
       length=100000
    else
       length=$((${scaffold_length}-${start}))
    fi

    #FASTASUBSEQ: we run this program to cut the scaffold around our hit so the exonerate can do a better alignment
    fastasubseq ${foldername}/${scaffold}/${foldername}_${scaffold}_fastafetch.fa ${start} ${length} > ${foldername}/${scaffold}/${foldername}_${scaffold}_fastasubseq.fa

    #EXONERATE: we run this program to align the query sequences of Danio rerio with the genome sequence of Ophiodon elongatus and obtain the genome sequence of the predicted protein
    echo "EXONERATE==============================================================================================================================================================================================="
    exonerate -m p2g --showtargetgff -q ${foldername}/${foldername}.fa -t ${foldername}/${scaffold}/${foldername}_${scaffold}_fastasubseq.fa | egrep -w "exon" > ${foldername}/${scaffold}/${foldername}_${scaffold}_exonerate.gff
    cat ${foldername}/${scaffold}/${foldername}_${scaffold}_exonerate.gff
    inici=$(cut -f 4 ${foldername}/${scaffold}/${foldername}_${scaffold}_exonerate.gff)
    final=$(cut -f 5 ${foldername}/${scaffold}/${foldername}_${scaffold}_exonerate.gff)
    echo $inici
    echo $final

    #FASTASEQFROMGFF: we run this program to extract the exon sequence in FASTA format
    fastaseqfromGFF.pl ${foldername}/${scaffold}/${foldername}_${scaffold}_fastasubseq.fa ${foldername}/${scaffold}/${foldername}_${scaffold}_exonerate.gff > ${foldername}/${scaffold}/${foldername}_${scaffold}_fastaseq.fa

    #FASTATRANSLATE: this progra, translates the genome to the predicted protein sequence
    fastatranslate -f ${foldername}/${scaffold}/${foldername}_${scaffold}_fastaseq.fa -F 1 > ${foldername}/${scaffold}/${foldername}_${scaffold}_fastatranslate.fa
    #The resulting sequences have the Sec residues marked as * so we change these symbols to X
    sed 's/*/X/g' -i ${foldername}/${scaffold}/${foldername}_${scaffold}_fastatranslate.fa

    #T-COFFEE: this program aligns the query sequences of Danio rerio with the predicted protein sequence of Ophiodon elongatus
    echo "t-COFFEE=============================================================================================================================================================================================="
    t_coffee ${foldername}/${foldername}.fa ${foldername}/${scaffold}/${foldername}_${scaffold}_fastatranslate.fa 2> /dev/null > ${foldername}/${scaffold}/${foldername}_${scaffold}_t_coffee.aln
    rm *.html | rm *.aln
    cat ${foldername}/${scaffold}/${foldername}_${scaffold}_t_coffee.aln

    #GENEWISE: we run this program to align the query sequences of Danio rerio with the genome sequence of Ophiodon elongatus and obtain the genome sequence of the predicted protein. The generated results have to be similar to the obtained with Exonerate
    echo "GENEWISE=============================================================================================================================================================================================="
    if [[ ${strand}=="reverse" ]]; then
      genewise -pep -pretty -cdna -gff -trev ${foldername}/${foldername}.fa ${foldername}/${scaffold}/${foldername}_${scaffold}_fastasubseq.fa | grep "cds" > ${foldername}/${scaffold}/${foldername}_${scaffold}_genewise.gff
      echo ${strand}
    elif [[ ${strand}=="forward" ]]; then
      genewise -pep -pretty -cdna -gff ${foldername}/${foldername}.fa ${foldername}/${scaffold}/${foldername}_${scaffold}_fastasubseq.fa | grep "cds" > ${foldername}/${scaffold}/${foldername}_${scaffold}_genewise.gff
      echo ${strand}
    fi
    cat ${foldername}/${scaffold}/${foldername}_${scaffold}_genewise.gff

  done
  #When this finishes, the program will rerun the "SCAFFOLD LOOP" if there is another scaffold to study or will print the command below and analyse the next protein on "Ophiodon_elongatus" folder

  echo "The protein ${foldername} has been analized"

done
